# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.14.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# +
# # %env CUDA_VISIBLE_DEVICES=0
# %env XLA_PYTHON_CLIENT_PREALLOCATE=false
# # %env XLA_PYTHON_CLIENT_MEM_FRACTION=.50
# # %env CUDA_LAUNCH_BLOCKING=1

import ipdb

import torch
import torch.nn.functional as F
import torch.autograd as autograd
import math
import numpy

import jax
import jax.numpy as jnp
from jax import grad, jit, vmap
from jax import random as random

from tqdm import trange


import argparse
import collections
import json
import os
import random as orandom
import sys
import time
import uuid

import numpy as np
import PIL
import torch
import torchvision
import torch.utils.data

from domainbed import datasets
from domainbed import hparams_registry
from domainbed import algorithms
from domainbed.lib import misc
from domainbed.lib.fast_data_loader import InfiniteDataLoader, FastDataLoader

from attrdict import AttrDict
import typer


# -

def jax_save(output_dir):
    save_dict = torch.load(os.path.join(output_dir, 'model.pkl'))
    args = AttrDict(save_dict['args'])
    hparams = AttrDict(save_dict['model_hparams'])

    orandom.seed(args.seed)
    np.random.seed(args.seed)
    torch.manual_seed(args.seed)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False

    if torch.cuda.is_available():
        device = "cuda"
    else:
        device = "cpu"

    if args.dataset in vars(datasets):
        dataset = vars(datasets)[args.dataset](args.data_dir,
                                               args.test_envs, hparams)
    else:
        raise NotImplementedError

    algorithm_class = algorithms.get_algorithm_class(args.algorithm)
    algorithm = algorithm_class(dataset.input_shape, dataset.num_classes,
                                len(dataset) - len(args.test_envs), hparams)
    algorithm.load_state_dict(save_dict["model_dict"])
    algorithm.to(device)

    os.makedirs(f"{args.output_dir}/embeddings", exist_ok=True)
    with torch.no_grad():
        for i, d in enumerate(dataset.datasets):
            loader = FastDataLoader(
                dataset=d,
                batch_size=hparams['batch_size'],
                num_workers=2)
                # num_workers=hparams.N_WORKERS)
            data = [(algorithm.featurizer(x.to(device)), y) for x, y in loader]
            xs = torch.cat([x for x, _ in data]).cpu()
            ys = torch.cat([y for _, y in data]).cpu()
            torch.save(
                xs, f"{args.output_dir}/embeddings/_DRODG_env_{i}_xs.pt")
            torch.save(
                ys, f"{args.output_dir}/embeddings/_DRODG_env_{i}_ys.pt")


# +
# output_dir = 'sweep_output/PACS/0af124f60da9e9ca8ed88f04934107e4'
# output_dir = 'train_output/PACS'
# jax_save(output_dir)

if __name__ == '__main__':
    typer.run(jax_save)
